package pl.net.linux.krzak.kchat.util;

/**
Class that convert protocol into XML representation and in return.

Example of use:

<pre>
    import pl.net.linux.krzak.kchat.util.kChatProtocolXMLWrapper;
    kChatProtocolXMLWrapper xmlWrapper = new kChatProtocolXMLWrapper("UTF-8","kChatProtocol.dtd");
    String xml = (xmlWrapper.plainToXML("*LOGIN|JEDEN|DWA|TRZY|"));
    System.out.println(xmlWrapper.xmlToPlain(xml));
</pre>
*/

public class kChatProtocolXMLWrapper {
	private String header;
	private String doctype;
	private String entityMain     = new String("<kChatProtocol>");
	private String entityMainC    = new String("</kChatProtocol>");
	private String entityCommand  = new String("<command>");
	private String entityCommandC = new String("</command>");
	private String entityParam    = new String("<param>");
	private String entityParamC   = new String("</param>");
	private kChatParse parse = new kChatParse();
	
	/**
	    Constructor
	    @param encoding - encoding name ex. UTF-8
	    @param dtd - DTD document to attach into xml if validation is required
	*/
	public kChatProtocolXMLWrapper(String encoding,String dtd) {
	header = new String("<?xml version=\"1.0\" encoding=\""+encoding+"\"?>");
	if (doctype != null)
	    doctype = new String("<!DOCTYPE kChatProtocol SYSTEM \""+dtd+"\">");
	}

	/**
	Convert protocol data into XML document
	<pre>
	    &lt!ELEMENT kChatProtocol (command,param*)>
	    &lt!ELEMENT command (#CDATA)>
	    &lt!ELEMENT param   (#CDATA)>
	</pre>
	@see kChatProtocolXMLWrapper#xmlToPlain
	*/
	public String plainToXML(String plain) {
	    int i = 1;
	    String xml = new String(header+doctype);
	    xml = xml + entityMain;
	    while (!(parse.getParam(plain,i).equals(""))) {
		String val = parse.getParam(plain,i);
		if ((i==1)&&(val.startsWith("*")))
		    xml = xml + entityCommand+val+entityCommandC;
		   else 
		    xml = xml + entityParam+val+entityParamC;
		i++;
		}
	    xml = xml + entityMainC;
	return(xml);
	}
	
	/** 
	Convert XML document into protocol data
	@see kChatProtocolXMLWrapper#plainToXML
	*/
	
	public String xmlToPlain(String xml) {
	String plain = new String();
	int start = xml.indexOf(entityMain);
	int end   = xml.indexOf(entityMainC,start+entityMain.length());
	
	/* find <command></command> */
	int start_c = xml.indexOf(entityCommand,start+entityMain.length());
	int end_c = xml.indexOf(entityCommandC,start_c+entityCommand.length());
	plain=xml.substring(start_c+entityCommand.length(),end_c)+"|";
	
	/* find all <param></param> */
	int start_p = start_c;
	int end_p = end_c;
	while ((start_p != -1)&&(end_p != -1)){
	    start_p = xml.indexOf(entityParam,end_p);
	    end_p = xml.indexOf(entityParamC,start_p+entityParam.length());
	    if ((start_p > 0)&&(end_p > 0))
		plain=plain+xml.substring(start_p+entityParam.length(),end_p)+"|";
	    }
	
	/* that's all folks */
	return(plain);
	}
}

