package pl.net.linux.krzak.kchat.util;

import java.util.Vector;
import java.awt.*;
import java.awt.event.*;

public class kChatTextArea extends Component implements AdjustmentListener {
    public static final int ACCEPT = 0;
    public static final int IGNORE = 1;
    private int pozX = 10;
    private int rows = 0;
    private int Width = 200;
    private int startAt = 0;
    private int linesAll = 0;
    private Graphics buforGraf;
    private Image bufor;
    private Image bgImage = null;
    private Dimension rozmiarBuf;
    private Vector messages = new Vector();
    private Scrollbar scroll;
    private boolean pressed = false; // true if the button is detented.
    private boolean over    = false; // true if the button is detented.
    private ClickListener clickListener = null;
    private MouseEvent me = null;
    private boolean ascroll = true;


    /** 
    Create object with default Width=200 and rows=2  
    */
    public kChatTextArea() {
 this(200,2); }

    /** 
	@param _Width Initial width of component
	@param _rows  Initial count of rows
    */
    public kChatTextArea(int _Width, int _rows) {
	this.Width = _Width;
	this.rows = _rows;
    }
    
    /**
	Set background image
    */
    public void setBackgroundImage(Image im) {
	if (im == null) return;
        bgImage = im;
    }    
    
    /**	
	Add ClickListener
	@see kChatTextArea#removeClickListener
    */
    public void addClickListener(ClickListener cl) { 
	pozX = 20;
	clickListener = cl; 
	enableEvents(AWTEvent.MOUSE_EVENT_MASK);
	}

    /**	
	Remove ClickListener
	@see kChatTextArea#addClickListener
    */
    public void removeClickListener() { 
	pozX = 10;
	disableEvents(AWTEvent.MOUSE_EVENT_MASK);
	clickListener = null; 
	}

    /**
     enable scroller support
    */
    public void addScroll(Scrollbar sc) {
	scroll = sc;
	scroll.addAdjustmentListener(this);
    }

    public void adjustmentValueChanged(AdjustmentEvent e) {
	    if (!ascroll) {
		int tmp = scroll.getMaximum() - 1 - scroll.getValue();
		if (tmp >= 0) startAt = tmp;
	    } else {
		startAt = e.getValue();
	    }
	    
	    this.paint(this.getGraphics());
    }
    
    /**
	get status automatic scrolling if new messages arrive
    */
    public boolean autoScroll() { return(ascroll); }

    /**
	Automatic scrolling if new messages arrive
	@param val default true
    */
    public void autoScroll(boolean val) { ascroll = val; }
    
    public int getrows() { return rows; }
    public int getOffset() { return pozX; }
    public void  setOffset(int off) { pozX = off; }


    /**
     set MessagesClass for given text
     height,width,lines etc..
     return preapred object
    */
    private MessagesClass calculateLines(String text) {
	MessagesClass tmp = new MessagesClass();
	FontMetrics fm = this.getGraphics().getFontMetrics(getFont());
	tmp.text = text;
	tmp.height = fm.getHeight();
	tmp.width  = fm.stringWidth(text);
	
	String lajn = new String();
	java.text.BreakIterator brit = java.text.BreakIterator.getWordInstance();
	brit.setText(text);
	int start = brit.first();
	for (int end = brit.next(); end != java.text.BreakIterator.DONE; start = end, end = brit.next()) {
	    if (fm.stringWidth(lajn+text.substring(start,end)) >= (getSize().width-pozX-5)) {
		tmp.linesV.addElement(lajn);
		lajn = "";
	    }
	lajn = lajn + text.substring(start,end);
	}
	tmp.linesV.addElement(lajn);
	return tmp;
    }


    /**
	Append text with specified colour into text area
	@param text Text to append
	@param color Text colour (ex. #1c1c1c)
	@see kChatTextArea#stringToColor
    */
    public void appendText(String text, String color) {
    if ((text == null) || (color == null)) return;

    text.trim();

    if (text.length()<=0) return;
    if (color.length()<=0) return;
    
    synchronized (this) {
    
      try {
        MessagesClass tmpmsg = new MessagesClass();

        tmpmsg = calculateLines(text);
        tmpmsg.color =
 stringToColor(color);
        tmpmsg.colortxt =
color;
	messages.addElement(tmpmsg);
	
	linesAll = linesAll + tmpmsg.linesV.size();
	if (!ascroll && (messages.size() > rows)) {
	    startAt = startAt + tmpmsg.linesV.size();
	}

        this.paint(this.getGraphics());
      } catch (StringIndexOutOfBoundsException e) {} 
    
    }
   
   }
   
   
   
    /**
	Remove text from textarea
	@param msg MessagesClass to remove
	@param color Text colour (ex. #1c1c1c)
	@see MessagesClass
    */
   public void removeMessage(MessagesClass msg) {
    linesAll = linesAll - msg.linesV.size();
    messages.removeElement(msg);
    this.paint(this.getGraphics());
   }





   public void paint(Graphics g) { update(g); }
   public synchronized void update(Graphics g) {
    if (Width<=0) return;

    if ((buforGraf == null)  || (getSize().width != rozmiarBuf.width)  || (getSize().height != rozmiarBuf.height)) {
	rozmiarBuf = getSize();
	bufor = createImage(getSize().width,getSize().height);
	buforGraf = bufor.getGraphics();
        buforGraf.setFont(getFont());
    }
    
    buforGraf.setColor(getBackground());
    buforGraf.fillRect(0, 0, getSize().width, getSize().height);
    if (bgImage != null) buforGraf.drawImage(bgImage,0,0,this);

    FontMetrics fm = getFontMetrics(buforGraf.getFont());
    int fontH = fm.getHeight();
    int pozY = 0;
    int lines = 0;
    rows = (getSize().height / fm.getHeight());
    pozY = (fontH * rows) - (fm.getMaxAscent()/2) + 1;
    
    Vector sensitiveRectangles = new Vector();
    boolean more = true;
    for(int i=(messages.size()-startAt-1);(i>=0) && (more);i--) {
	MessagesClass tmpmsg = (MessagesClass)messages.elementAt(i);
	
	buforGraf.setColor(tmpmsg.color);

        tmpmsg.rectangle.x = pozX;
	tmpmsg.rectangle.y = pozY;
	
//	for (int l=0;(l<tmpmsg.linesV.size()) && (more);l++) {
	for (int l=tmpmsg.linesV.size()-1;(l>=0) && (more);l--) {
	
	    buforGraf.drawString((String)tmpmsg.linesV.elementAt(l),pozX,pozY);
	    pozY=pozY-tmpmsg.height;

	    tmpmsg.rectangle.setSize(tmpmsg.width,tmpmsg.rectangle.y-pozY);
	    tmpmsg.rectangle.y = tmpmsg.rectangle.y - tmpmsg.rectangle.height ;
	    sensitiveRectangles.addElement(tmpmsg);
	    
	    if (lines++ > rows) more = false;

	}

    }

    if (clickListener != null) {
     for (int i=0;i<sensitiveRectangles.size();i++) {
	MessagesClass msgtmp = (MessagesClass)sensitiveRectangles.elementAt(i);
        if (clickListener != null) { 
    	    buforGraf.setColor(Color.red);
    	    buforGraf.fillRoundRect(msgtmp.rectangle.x-pozX, msgtmp.rectangle.y+3, pozX-1, msgtmp.rectangle.height-3,5,5);
	}

        if (pressed && (me != null) && insideRectangle(msgtmp.rectangle,me.getPoint(),IGNORE)) {
	    pressed=false;
    	    clickListener.selectedItem(msgtmp,IGNORE);
	}

        if (pressed && (me != null) && insideRectangle(msgtmp.rectangle,me.getPoint(),ACCEPT)) {
	    pressed=false;
    	    clickListener.selectedItem(msgtmp,ACCEPT);
	}
     }

    }
    
    scroll.setMaximum(linesAll-rows+1);
    g.drawImage(bufor, 0, 0, this);
    }


    
    /**
	Convert string color represetation into proper object
	@param color Color value like #1c1c1c
    */
    public Color stringToColor(String color)
 {
 return new Color(1).decode(color); }
    
    public Dimension getMinimumSize() { return new Dimension(100, 50); }
    
    public Dimension getPreferredSize() {
	FontMetrics fm = getFontMetrics(super.getFont());
	startAt = 0;
        int W = Width;
        int H = (fm.getHeight() * rows);
        return new Dimension(W,H);
    }

   private boolean insideRectangle(Rectangle rect, Point point, int what) {
	if (what == ACCEPT) {
	    return ( 
	     (point.x > rect.x) && 
	     (point.x < (rect.x+rect.width)) &&
	     (point.y > rect.y) &&
	     (point.y < (rect.y+rect.height)) );
	    }
	if (what == IGNORE) {
	    return ( (point.x > rect.x-pozX) && 
	     (point.x < (rect.x)) && 
	     (point.y > rect.y) && 
	     (point.y < (rect.y + rect.height)) );
	}
	return false;
   }

   public void processMouseEvent(MouseEvent e) {
       me = e;
       switch(e.getID()) {
          case MouseEvent.MOUSE_PRESSED:
	    pressed = true;
	    this.paint(this.getGraphics());
	    break;
          case MouseEvent.MOUSE_RELEASED:
	    if(pressed == true) {
		pressed = false;
		this.paint(this.getGraphics());
	    }
	    break;
          case MouseEvent.MOUSE_ENTERED:
	    if (over == false) {
		over = true;
		this.paint(this.getGraphics());
		}
	    break;
          case MouseEvent.MOUSE_EXITED:
	    if (over == true) { 
		over = false;
		this.paint(this.getGraphics());
	    }
	    if(pressed == true) {
		pressed = false;
		this.paint(this.getGraphics());
	    }
	    break;
       }
       super.processMouseEvent(e);
   }
    

}
